# -*- coding: utf-8 -*-
#
# File: .py
#
# Copyright (c) InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__author__    = """Stefan Eletzhofer <stefan.eletzhofer@inquant.de>"""
__docformat__ = 'plaintext'
__revision__  = "$Revision: 62293 $"
__version__   = '$Revision: 62293 $'[11:-2]

from zope import component
from zope import interface

from zope.publisher.interfaces import NotFound
from zope.publisher.interfaces import IPublishTraverse
from zope.publisher.http import IHTTPRequest

from ZPublisher.BaseRequest import DefaultPublishTraverse

from Acquisition import aq_inner, aq_base
from Products.CMFCore.utils import getToolByName

from inquant.contentmirror.base.interfaces import IMirrorContentProvider
from inquant.contentmirror.base.interfaces import IMirrorContentLocator
from inquant.contentmirror.base.utils import give_new_context
from inquant.contentmirror.base.utils import info, debug

class MirrorObjectTraverser(object):
    """
    A traverser which tries to locate mirrored objects. If such a object can be
    located (by querying the IMirrorObjectLocator), then the object returned by
    the locator is inserted into the context's acquisition chain.
    """
    component.adapts(IMirrorContentProvider, IHTTPRequest)
    interface.implements(IPublishTraverse)
    def __init__(self,context,request):
        self.locator = IMirrorContentLocator(context)
        self.default = DefaultPublishTraverse(context,request)
        self.context = context
        self.request = request

    def publishTraverse(self, request, name):
        try:
            obj = self.default.publishTraverse(request, name)
            return obj
        except (NotFound, AttributeError), e:
            context = aq_inner(self.context)
            debug("MirrorObjectTraverser: default traverser returned NotFound. %s -> %s" % (self.context,name))

            obj = self.locator.locate(name)
            if obj is None:
                raise e

            obj = give_new_context(obj, context)
            debug("MirrorObjectTraverser: ctx %s, name %s -> %s" % (context,name,obj))
            return obj


# vim: set ft=python ts=4 sw=4 expandtab :



# -*- coding: utf-8 -*-
#
# File: test_doctests.py
#
# Copyright (c) InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__author__    = """Stefan Eletzhofer <stefan.eletzhofer@inquant.de>"""
__docformat__ = 'plaintext'
__revision__  = "$Revision: 57037 $"
__version__   = '$Revision: 57037 $'[11:-2]

import unittest
import doctest

from Testing import ZopeTestCase as ztc

from inquant.contentmirror.base.tests import base

doctests = "README.rst".split()

def test_suite():
    return unittest.TestSuite(

        [ ztc.ZopeDocFileSuite(
            dtfile, package='inquant.contentmirror.base',
            test_class=base.CMFunctionalTestCase,
            optionflags=(doctest.ELLIPSIS | 
                         doctest.NORMALIZE_WHITESPACE))

            for dtfile in doctests ]
        )

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')

# vim: set ft=python ts=4 sw=4 expandtab :



# -*- coding: utf-8 -*-
#
# File: skeleton.py
#
# Copyright (c) InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__author__    = """Stefan Eletzhofer <stefan.eletzhofer@inquant.de>"""
__docformat__ = 'plaintext'
__revision__  = "$Revision: 62293 $"
__version__   = '$Revision: 62293 $'[11:-2]

import unittest

from zope import component
from zope import interface
from zope.publisher.browser import TestRequest
from zope.publisher.interfaces import IPublishTraverse
from zope.publisher.interfaces.http import IHTTPRequest

from zope.app.testing import ztapi

from zope.traversing.interfaces import ITraversable, TraversalError

from inquant.contentmirror.base.interfaces import IMirrorUIDManager
from inquant.contentmirror.base.interfaces import IMirrorContentProvider
from inquant.contentmirror.base.traverser import MirrorObjectTraverser
from inquant.contentmirror.base.locator import UIDLocator
from inquant.contentmirror.base.tests import base

class TestUIDManager(base.CMTestCase):
    def testAdapterLookup(self):
        self.failUnless(component.queryAdapter(self.folder, IMirrorUIDManager))

    def testUidManager(self):
        manager = IMirrorUIDManager(self.folder)
        manager.set("muh", "17")
        self.assertEqual(manager.get("muh"), "17")

        manager.remove("muh")
        self.failUnless("muh" not in manager.storage.keys())

class TestLocator(base.CMTestCase):
    def afterSetUp(self):
        self.setRoles(('Manager',))
        self.folder.invokeFactory("Folder", "f1")
        self.folder.invokeFactory("Folder", "f2")
        self.f1 = self.folder.f1
        self.f2 = self.folder.f2
        self.f1.invokeFactory("Document", "doc", title="Muh")

        # mirror "doc" which is originally in f1 only to f2 also
        manager = IMirrorUIDManager(self.f2)
        manager.set("doc", self.f1.doc.UID())

    def testLocator(self):
        adapter = UIDLocator(self.f2)
        self.failUnless(adapter.locate("doc"))

class TestTraverser(base.CMTestCase):
    def afterSetUp(self):
        self.setRoles(('Manager',))
        self.folder.invokeFactory("Folder", "f1")
        self.folder.invokeFactory("Folder", "f2")
        self.f1 = self.folder.f1
        self.f2 = self.folder.f2
        self.f1.invokeFactory("Document", "doc", title="Muh")
        self.request = TestRequest(environ={'URL': 'http://127.0.0.1'})

        # mirror "doc" which is originally in f1 only to f2 also
        manager = IMirrorUIDManager(self.f2)
        manager.set("doc", self.f1.doc.UID())

        component.provideAdapter(MirrorObjectTraverser)

    def testTraversalAdapterLookupNoMirrorProvider(self):
        self.failIf(IMirrorContentProvider.providedBy(self.f2))
        self.failIf(component.queryMultiAdapter((self.f2,self.request), IPublishTraverse))

    def testTraversalAdapterLookup(self):
        interface.alsoProvides(self.f2,IMirrorContentProvider)
        self.failUnless(component.getMultiAdapter((self.f2,self.request), IPublishTraverse))
        interface.noLongerProvides(self.f2,IMirrorContentProvider)

    def testTraversal(self):
        gsm = component.getGlobalSiteManager()
        interface.alsoProvides(self.f2,IMirrorContentProvider)
        adapter = component.getMultiAdapter((self.f2,self.request), IPublishTraverse)
        self.failUnless(adapter.publishTraverse(self.request, "doc"))
        interface.noLongerProvides(self.f2,IMirrorContentProvider)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestUIDManager))
    suite.addTest(unittest.makeSuite(TestLocator))
    suite.addTest(unittest.makeSuite(TestTraverser))
    return suite

# vim: set ft=python ts=4 sw=4 expandtab :




# -*- coding: utf-8 -*-
#
# File: manager.py
#
# Copyright (c) InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__author__    = """Stefan Eletzhofer <stefan.eletzhofer@inquant.de>"""
__docformat__ = 'plaintext'
__revision__  = "$Revision: 62293 $"
__version__   = '$Revision: 62293 $'[11:-2]

from zope import component
from zope import interface

from zope.annotation.interfaces import IAnnotations, IAttributeAnnotatable
from zope.app.component.hooks import getSite
from zope.event import notify

from persistent.dict import PersistentDict
from Acquisition import aq_inner

from inquant.contentmirror.base.interfaces import IContentMirroredEvent, IContentNoLongerMirroredEvent
from inquant.contentmirror.base.interfaces import IMirrorAddedEvent, IMirrorRemovedEvent
from inquant.contentmirror.base.interfaces import IMirrorWillBeAddedEvent, IMirrorWillBeRemovedEvent
from inquant.contentmirror.base.interfaces import IMirroredContent
from inquant.contentmirror.base.interfaces import IMirrorReferenceManager
from inquant.contentmirror.base.interfaces import IMirrorUIDManager
from inquant.contentmirror.base.interfaces import IMirrorContentProvider
from inquant.contentmirror.base.interfaces import IMirroredContentManager

from inquant.contentmirror.base.utils import give_new_context, info, debug

class ContentMirrored(object):
    interface.implements(IContentMirroredEvent)
    def __init__(self, obj):
        self.object = obj

class ContentNoLongerMirrored(object):
    interface.implements(IContentNoLongerMirroredEvent)
    def __init__(self, obj):
        self.object = obj

class MirrorAdded(object):
    interface.implements(IMirrorAddedEvent)
    def __init__(self, obj, container):
        self.object = obj
        self.container = container

class MirrorRemoved(MirrorAdded):
    interface.implements(IMirrorRemovedEvent)

class MirrorWillBeAdded(MirrorAdded):
    interface.implements(IMirrorWillBeAddedEvent)

class MirrorWillBeRemoved(MirrorRemoved):
    interface.implements(IMirrorWillBeRemovedEvent)

class AnnotationUIDManager(object):

    KEY="inquant.contentmirror.uidmanager"

    interface.implements(IMirrorUIDManager)
    component.adapts(IAttributeAnnotatable)

    def __init__(self, context):
        self.context = context
        s = IAnnotations(context)
        self.storage = s.setdefault(self.KEY, PersistentDict())

    def items(self):
        return self.storage.items()

    def keys(self):
        return self.storage.keys()

    def set(self, key, uid):
        self.storage[key] = uid

    def get(self, key, default=None):
        return self.storage.get(key,default)

    def remove(self,key):
        del self.storage[key]

class DefaultReferenceManager(object):
    """ this default implementation stores information on where a content is
    mirrored on the conten itself using annotations """

    component.adapts(IMirroredContent)
    interface.implements(IMirrorReferenceManager)

    KEY="inquant.contentmirror.refmanager.mirrors"
    KEY_OPATH="inquant.contentmirror.refmanager.original"

    def __init__(self, context):
        self.context = aq_inner(context)

    def _make_key(self, obj, container):
        return "%s/%s" % ("/".join(container.getPhysicalPath()), obj.getId())

    @property
    def storage(self):
        s = IAnnotations(self.context)
        return s[self.KEY]

    @property
    def opath(self):
        s = IAnnotations(self.context)
        return s[self.KEY_OPATH]

    def initialize(self, original):
        s = IAnnotations(self.context)
        s[self.KEY] = PersistentDict()
        s[self.KEY_OPATH] = "/".join(original.getPhysicalPath())

    def deinitialize(self, original):
        s = IAnnotations(self.context)
        del s[self.KEY]
        del s[self.KEY_OPATH]

    def update(self, original):
        info("update: old %s" % self.opath )
        s = IAnnotations(self.context)
        s[self.KEY_OPATH] = "/".join(original.getPhysicalPath())
        info("update: new %s" % self.opath )

    def add(self, obj, container):
        notify(MirrorWillBeAdded(obj,container))
        key = self._make_key(obj,container)
        d = dict(
                container_uid  = container.UID(),
                container_path = "/".join(container.getPhysicalPath()),
        )
        self.storage[key] = d
        notify(MirrorAdded(obj,container))
        info("DefaultReferenceManager: added %s: %s" % (key, d))

    def remove(self, obj, container):
        notify(MirrorWillBeRemoved(obj,container))
        key = self._make_key(obj,container)
        del self.storage[key]
        debug("DefaultReferenceManager: removed %s" % key)
        notify(MirrorRemoved(obj,container))

        # inform caller if this was the last mirror
        if not len(self.storage.keys()):
            return True
        else:
            return False

    def isMirror(self, obj, container):
        """ return True if the object given is a mirror """
        key = self._make_key(obj,container)
        ret = key in self.storage.keys()
        debug("DefaultReferenceManager.isMirror(): key=%s -> %s" % (key, ret))
        return key in self.storage.keys()

    def getOriginal(self):
        site = getSite()
        return site.restrictedTraverse(self.opath)

    def items(self):
        return self.storage.items()



class DefaultMirrorManager(object):
    interface.implements(IMirroredContentManager)

    def addMirror(self, obj, container):
        # mark the container if not already marked
        if not IMirrorContentProvider.providedBy(container):
            interface.alsoProvides(container,IMirrorContentProvider)

        # give obj a new context
        new_obj = give_new_context(obj, container)

        # add the UID of the object to be mirrored to the UID manager
        uid_manager = IMirrorUIDManager(container)
        uid_manager.set(obj.getId(), obj.UID())

        if not IMirroredContent.providedBy(obj):
            # content is not mirrored already. Mark it.
            interface.alsoProvides(obj, IMirroredContent)
            notify(ContentMirrored(obj))

        # get a ref manager and add a reference
        refmgr = IMirrorReferenceManager(obj)
        refmgr.add(new_obj, container)

        # reindex object
        new_obj.reindexObject()

        info("DefaultMirrorManager: added mirror of %s to %s" % ( new_obj.absolute_url(), container.absolute_url()))

    def removeMirror(self, obj, container):
        if not IMirrorContentProvider.providedBy(container):
            return False

        if not IMirroredContent.providedBy(obj):
            return False

        # remove obj's UID form the manager
        uid_manager = IMirrorUIDManager(container)
        if not uid_manager.get(obj.getId()):
            return False

        # adjust acquisition chain
        new_obj = give_new_context(obj, container)

        uid_manager.remove(obj.getId())

        # get a ref manager and add a reference
        refmgr = IMirrorReferenceManager(obj)
        if refmgr.remove(new_obj, container):
            # last mirror removed, notify and unmark **OBJECT**.
            original = refmgr.getOriginal()
            notify(ContentNoLongerMirrored(original))
            interface.noLongerProvides(original, IMirroredContent)

        # if no more mirrors available, unmark **CONTAINER**
        if not uid_manager.keys():
            interface.noLongerProvides(container, IMirrorContentProvider)

        info("DefaultMirrorManager: removed mirror of %s in %s" % (new_obj.absolute_url(), container.absolute_url()))
        return True

# vim: set ft=python ts=4 sw=4 expandtab :



# -*- coding: utf-8 -*-
#
# File: test_refmgr.py
#
# Copyright (c) InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__author__    = """Stefan Eletzhofer <stefan.eletzhofer@inquant.de>"""
__docformat__ = 'plaintext'
__revision__  = "$Revision: 62293 $"
__version__   = '$Revision: 62293 $'[11:-2]

import unittest

from zope import component
from zope import interface

from zope.app.testing import ztapi

from inquant.contentmirror.base.interfaces import IMirroredContentManager
from inquant.contentmirror.base.interfaces import IMirrorContentLocator
from inquant.contentmirror.base.interfaces import IMirrorReferenceManager
from inquant.contentmirror.base.manager import DefaultReferenceManager
from inquant.contentmirror.base.traverser import MirrorObjectTraverser
from inquant.contentmirror.base.tests import base

class TestRefmgr(base.CMTestCase):
    def afterSetUp(self):
        self.setRoles(('Manager',))
        self.folder.invokeFactory("Folder", "f1")
        self.folder.invokeFactory("Folder", "f2")
        self.f1 = self.folder.f1
        self.f2 = self.folder.f2
        self.f1.invokeFactory("Document", "doc", title="Muh")

        component.provideAdapter(MirrorObjectTraverser)
        component.provideAdapter(DefaultReferenceManager)

        # mirror "doc" which is originally in f1 only to f2 also
        manager = component.getUtility(IMirroredContentManager)
        manager.addMirror(self.f1.doc, self.f2)

        self.original = self.f1.doc
        self.mirror = IMirrorContentLocator(self.f2).locate("doc")

    def testRefMgrLookup(self):
        self.failUnless(IMirrorReferenceManager(self.original))
        self.failUnless(IMirrorReferenceManager(self.mirror))

    def testRefMgrKeys(self):
        rm = IMirrorReferenceManager(self.mirror)
        self.assertEqual(len(rm.storage.keys()), 1)
        self.assertEqual(rm.storage.keys(), ['/plone/Members/test_user_1_/f2/doc',])

    def testIsMirror(self):
        rm = IMirrorReferenceManager(self.mirror)
        self.failUnless(rm.isMirror(self.mirror, self.f2))

        rm = IMirrorReferenceManager(self.original)
        self.failUnless(rm.isMirror(self.mirror, self.f2))

    def testGetOriginal(self):
        rm = IMirrorReferenceManager(self.mirror)
        self.assertEqual(self.original.getPhysicalPath(), rm.getOriginal().getPhysicalPath())


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestRefmgr))
    return suite

# vim: set ft=python ts=4 sw=4 expandtab :




# -*- coding: utf-8 -*-
#
# File: .py
#
# Copyright (c) InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__author__    = """Stefan Eletzhofer <stefan.eletzhofer@inquant.de>"""
__docformat__ = 'plaintext'
__revision__  = "$Revision: 62293 $"
__version__   = '$Revision: 62293 $'[11:-2]


from zope import interface

from zope.component.interfaces import IObjectEvent

class IMirrorContentProvider(interface.Interface):
    """ marker interface for objects providing mirror content """

class IMirroredContent(interface.Interface):
    """ marker interface for content which is in fact mirrored somewhere """

class IMirroredContentManager(interface.Interface):
    """ manager which mirrors content """

    def addMirror(obj, container):
        """ add a mirror object to a container """

    def removeMirror(obj, container):
        """ remove the object mirror form the container
            returns True if mirror was removed.
            returns False if object was not mirrored here.
        """

class IMirrorReferenceManager(interface.Interface):
    """ manages references of mirrors of a object """

    def add(obj, container):
        """ record a reference of the object to the container """

    def remove(obj, container):
        """ record the fact that 'obj' is no longer mirrored in 'container' """

    def initialize(original):
        """ set information about the original object """

    def update(original):
        """ update information about the original object """

    def deinitialize(original):
        """ content no longer mirrored, record the fact """

    def getOriginal():
        """ return the original object """

    def isMirror(obj, container):
        """ return True if obj is a mirror in container """

    def items():
        """ return a iterable of dicts { container_uid, container_path } which
            record the places this object is mirrored to. """

class IContentMirroredEvent(IObjectEvent):
    pass

class IContentNoLongerMirroredEvent(IObjectEvent):
    pass

class IMirrorWillBeAddedEvent(IObjectEvent):
    container = interface.Attribute(u"the container the mirror is added to")
    object    = interface.Attribute(u"the object to be mirrored")

class IMirrorAddedEvent(IObjectEvent):
    container = interface.Attribute(u"the container the mirror is added to")
    object    = interface.Attribute(u"the object to be mirrored")

class IMirrorWillBeRemovedEvent(IObjectEvent):
    container = interface.Attribute(u"the container the mirror is removed from")
    object    = interface.Attribute(u"the mirrored object")

class IMirrorRemovedEvent(IObjectEvent):
    container = interface.Attribute(u"the container the mirror is removed from")
    object    = interface.Attribute(u"the mirrored object")

class IMirrorContentLocator(interface.Interface):
    """ an adapter which is able to lookup and return a content
        object.
        rhe content object returned will be inserted (mirrored) at the
        adapter's context. """

    def locate(name):
        """ locate a content object identified by the key "name"  and
            return it """

class IMirrorUIDManager(interface.Interface):
    """
    Storage for key->UID mappings.
    """

    # XXX: this should behave like a mapping.

    def keys(self):
        """ """

    def get(key, default=None):
        """ return the UID stored for key """

    def set(key, uid):
        """ store a uid for the given key """

    def remove(key):
        """ remove the key from the storage """

# vim: set ft=python ts=4 sw=4 expandtab :



# -*- coding: utf-8 -*-
#
# File: base.py
#
# Copyright (c) InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__author__    = """Stefan Eletzhofer <stefan.eletzhofer@inquant.de>"""
__docformat__ = 'plaintext'
__revision__  = "$Revision: 57037 $"
__version__   = '$Revision: 57037 $'[11:-2]

from Products.Five import zcml
from Products.Five import fiveconfigure

from Testing import ZopeTestCase as ztc

from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import onsetup

@onsetup
def setup_contentmirror():
    """Set up the additional products
    """

    fiveconfigure.debug_mode = True
    import inquant.contentmirror.base
    zcml.load_config('configure.zcml', inquant.contentmirror.base)
    fiveconfigure.debug_mode = False

    ztc.installPackage('inquant.contentmirror.base')

setup_contentmirror()
ptc.setupPloneSite(products=['inquant.contentmirror.base'])


class CMTestCase(ptc.PloneTestCase):
    """Base class used for test cases
    """
    def create(self, context, what, name):
        context.invokeFactory(what, name, title=name)
        object = context.get(name)
        object.processForm()
        return object

class CMFunctionalTestCase(ptc.FunctionalTestCase):
    """Test case class used for functional (doc-)tests
    """

# vim: set ft=python ts=4 sw=4 expandtab :





"""Main product initializer
"""

from zope.i18nmessageid import MessageFactory


# Define a message factory for when this product is internationalised.
# This will be imported with the special name "_" in most modules. Strings
# like _(u"message") will then be extracted by i18n tools for translation.

CMMessageFactory = MessageFactory('inquant.contentmirror.base')

def initialize(context):
    """Intializer called when used as a Zope 2 product.
    
    This is referenced from configure.zcml. Regstrations as a "Zope 2 product" 
    is necessary for GenericSetup profiles to work, for example.
    
    Here, we call the Archetypes machinery to register our content types 
    with Zope and the CMF.
    """




from setuptools import setup, find_packages

version = '0.2'

setup(name='inquant.contentmirror.base',
      version=version,
      description="mirror plone content transparently in a plone site",
      long_description=file("inquant/contentmirror/base/README.rst").read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='',
      author='Stefan Eletzhofer',
      author_email='stefan.eletzhofer@inquant.de',
      url='https://svn.plone.org/svn/collective/inquant.contentmirror.base/trunk',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['inquant', 'inquant.contentmirror'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



# -*- coding: utf-8 -*-
#
# File: adapter.py
#
# Copyright (c) InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__author__    = """Stefan Eletzhofer <stefan.eletzhofer@inquant.de>"""
__docformat__ = 'plaintext'
__revision__  = "$Revision: 62293 $"
__version__   = '$Revision: 62293 $'[11:-2]

import logging

from zope import component
from zope import interface

from zope.annotation.interfaces import IAnnotations, IAttributeAnnotatable
from persistent.dict import PersistentDict

from Acquisition import aq_inner
from Products.CMFCore.utils import getToolByName

from inquant.contentmirror.base.interfaces import IMirrorContentProvider
from inquant.contentmirror.base.interfaces import IMirrorContentLocator
from inquant.contentmirror.base.interfaces import IMirrorUIDManager

from inquant.contentmirror.base.utils import info, debug, error


class UIDLocator(object):
    interface.implements(IMirrorContentLocator)
    component.adapts(IMirrorContentProvider)

    def __init__(self,context):
        self.context = context
        self.uic = getToolByName(context, "uid_catalog")

    def locate(self, name):
        context = aq_inner(self.context)
        debug("UIDTraverser: trying to locate %s (context %s)" % (name, self.context))
        manager = component.queryAdapter(context, IMirrorUIDManager)
        if not manager:
            error("UIDTraverser: no UID manager")
            return None

        uid = manager.get(name, None)
        if not uid:
            error("UIDTraverser: no UID found for '%s'" % name)
            return None

        debug("UIDTraverser: UID: %s" % uid)

        # fetch the object via UID
        res = self.uic(UID=uid)
        if not len(res):
            debug("UIDTraverser: UID lookup failed")
            return None

        for brain in res:
            obj = brain.getObject()
            if obj is not None:
                return obj


# vim: set ft=python ts=4 sw=4 expandtab :



# -*- coding: utf-8 -*-
#
# File: .py
#
# Copyright (c) InQuant GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__author__    = """Stefan Eletzhofer <stefan.eletzhofer@inquant.de>"""
__docformat__ = 'plaintext'
__revision__  = "$Revision: 62293 $"
__version__   = '$Revision: 62293 $'[11:-2]

from zope.component import adapter

from zope.app.component.hooks import getSite
from zope.app.container.interfaces import IObjectRemovedEvent, IObjectMovedEvent

from inquant.contentmirror.base.interfaces import IMirroredContent
from inquant.contentmirror.base.interfaces import IContentMirroredEvent, IContentNoLongerMirroredEvent
from inquant.contentmirror.base.interfaces import IMirrorAddedEvent, IMirrorRemovedEvent
from inquant.contentmirror.base.interfaces import IMirrorWillBeAddedEvent, IMirrorWillBeRemovedEvent
from inquant.contentmirror.base.interfaces import IMirrorReferenceManager

from inquant.contentmirror.base.utils import give_new_context
from inquant.contentmirror.base.utils import info, debug

from Products.CMFCore.utils import getToolByName

def unindex_object(site, obj):
    pc = getToolByName(site, "portal_catalog")
    pc.unindexObject(obj)

@adapter(IMirroredContent, IObjectMovedEvent)
def ObjectMoved(obj, event):
    info("EVENT ObjectMoved: %s" % event.object.absolute_url())
    IMirrorReferenceManager(event.object).update(event.object)

@adapter(IContentMirroredEvent)
def ContentMirrored(event):
    info("EVENT ContentMirrored: %s" % event.object.absolute_url())
    IMirrorReferenceManager(event.object).initialize(event.object)

@adapter(IContentNoLongerMirroredEvent)
def ContentNoLongerMirrored(event):
    info("EVENT ContentNoLongerMirrored: %s" % event.object.absolute_url())
    IMirrorReferenceManager(event.object).deinitialize(event.object)


@adapter(IMirroredContent, IObjectRemovedEvent)
def ObjectRemoved(obj, event):
    """ An object is going to be removed which happens to be mirrored. If the object is
    not a mirror, remove its mirrors """
    debug("Mirror Object removed: %s" % (obj.absolute_url()))
    refmgr = IMirrorReferenceManager(event.object)
    site = getSite()
    if not refmgr.isMirror(event.object, event.oldParent):
        # we're deleting the original. Delete all mirrors, too.
        for path, mi in refmgr.items():
            container = site.restrictedTraverse(mi["container_path"])
            mirror = give_new_context(obj, container)
            unindex_object(site, mirror)
            debug("Mirror Object removed: removed mirror: %s" % mirror.absolute_url())


@adapter(IMirrorWillBeRemovedEvent)
def MirrorRemoved(event):
    """ An mirrored content is about to be removed. Clean up the catalog. """
    debug("Mirror will be removed: event=%s container=%s object=%s" % (
        event,
        event.container.absolute_url(),
        event.object.absolute_url()
        ))
    refmgr = IMirrorReferenceManager(event.object)
    if refmgr.isMirror(event.object, event.container):
        unindex_object(getSite(), event.object)
        debug("Mirror removed: uncataloged object %s" % (event.object.absolute_url()))

# vim: set ft=python ts=4 sw=4 expandtab :
